#include <kirigami/kirigami2_export.h>

#if KIRIGAMI2_ENABLE_DEPRECATED_SINCE(5, 91)
#   include <kirigami/units.h>
#   if KIRIGAMI2_DEPRECATED_WARNINGS_SINCE >= 0x055b00
#       pragma message("Deprecated header. Since 5.91, use #include <kirigami/units.h> instead")
#   endif
#else
#   error "Include of deprecated header is disabled"
#endif
